#
#  Copyright 2002-2003 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

set MxOptimize::allow_fitted_maximum 1

proc create_optimize_gui { record_list } {
	frame .title
	pack .title -before .message_log
	
	label .title.text -text "Sector 22-ID Beamline Optimization"
	pack .title.text
	
	#-----------------------------------------------------------

	set statuswidth 30
	
	frame .sep1 -height 2 -borderwidth 1 -relief sunken
	pack .sep1 -expand yes -fill x -pady 5 -before .message_log
	
	frame .main
	pack .main -padx 10 -before .message_log
	
	set row 0
	
	label .main.mono_header -text Monochromator
	label .main.status_header -text Status
	label .main.old_header -text "Old Position"
	label .main.new_header -text "New Position"

	grid .main.mono_header         -row $row -column 0 -sticky w
	grid .main.status_header       -row $row -column 2
	grid .main.old_header          -row $row -column 3 -sticky w
	grid .main.new_header          -row $row -column 4 -sticky w
	
	incr row

	set tune [ $record_list get_record "tune" ]

	set thisfield .main.tune
	
	button ${thisfield}_button -text "Optimize Tune" \
			-background limegreen -command \
		[ itcl::code optimize_tune $record_list $tune $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $tune ]
	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row

	#-----------------------------------------------------------
	
	label .main.blank2
	grid .main.blank2
	
	incr row
	
	label .main.slits -text Slits
	grid .main.slits -sticky w
	
	incr row

	set gs_hor [ $record_list get_record "gs_hor" ]

	set thisfield .main.gs_hor
	
	button ${thisfield}_button -text "Optimize Horizontal Slit" \
			-background limegreen -command \
		[ itcl::code optimize_gs_hor $record_list $gs_hor $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $gs_hor ]
	label .main.gs_hor_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	#-----------------------------------------------------------
	
	set gs_vert [ $record_list get_record "gs_vert" ]

	set thisfield .main.gs_vert

	button ${thisfield}_button -text "Optimize Vertical Slit" \
	    -background limegreen -command \
	    [ itcl::code optimize_gs_vert $record_list $gs_vert $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text \
		[ optimize_get_position $gs_vert ]

	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	#-----------------------------------------------------------
	
	frame .sep2 -height 2 -borderwidth 1 -relief sunken
	pack .sep2 -expand yes -fill x -pady 8 -before .message_log
	
	frame .bottom
	pack .bottom -before .message_log
	
	button .bottom.dismiss -text Dismiss -command exit
	pack .bottom.dismiss

	frame .sep3 -height 2 -borderwidth 1 -relief sunken
	pack .sep3 -expand yes -fill x -pady 8 -before .message_log
	
	wm deiconify .
}

proc disable_buttons {} {
	.main.tune_button configure -state disabled
	.main.gs_hor_button configure -state disabled
	.main.gs_vert_button configure -state disabled
}

proc enable_buttons {} {
	.main.tune_button configure -state normal
	.main.gs_hor_button configure -state normal
	.main.gs_vert_button configure -state normal
}

proc optimize_tune { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_gs_hor { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}


proc optimize_gs_vert { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc ser22id_start_final_move { motor new_position } {

	$motor move_absolute $new_position

	set motor_list [ list $motor ]

	set motor_name [ $motor get_field "name" ]

	if { $motor_name == "gs_hor" } {
		set second_motor \
			[ $MxOptimize::record_list get_record "g_hor" ]

		set offset_record \
			[ $MxOptimize::record_list get_record "g_hor_offset" ]

	} elseif { $motor_name == "gs_vert" } {
		set second_motor \
			[ $MxOptimize::record_list get_record "g_vert" ]

		set offset_record \
			[ $MxOptimize::record_list get_record "g_vert_offset" ]
	} else {
		return $motor_list
	}

	set offset [ $offset_record receive_variable ]

	set second_position [ expr ( $new_position + $offset ) ]

	$second_motor move_absolute $second_position

	lappend motor_list $second_motor

	return $motor_list
}

